import 'package:flutter/material.dart';
import 'package:fluttered/architecture.dart';
import 'package:provider_view_controller_example/controllers/pvc_demo_immutable_controller.dart';

class StatefulImmutablePage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return StatefulImmutablePvc(
      controller: PVCDemoImmutableController(),
      onModelReady: (controller) => controller.onInitView(),
      builder: (context, controller, child) => Scaffold(
        appBar: AppBar(
          title: Text('StatefulImmutablePVC'),
        ),
        body: Center(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Text(
                'Notice the counter value starts with 55 instead of 0. This is because we are invoking the "controller.onInitView()" method which sets the initial value of "counter" to 55, by passing "controller.onInitView()" as "onModelReady" parameter.',
                style: TextStyle(fontSize: 18),
              ),
              SizedBox(height: 100),
              Text(
                'You have pushed the button this many times:',
                style: TextStyle(fontSize: 22),
              ),
              Text(
                '${controller.counter}',
                style: Theme.of(context).textTheme.display1,
              ),
              SizedBox(height: 100),
              Text(
                'Tapping the button will not update the UI even though "controller.incrementCounter()" method is changing state by calling "notifyListeners" because, "StatelessImutablePVC" creates a "Provider". A "Provider" does not notify it\'s consumers about changes, and thus the widget tree do not rebuild itself.',
                style: TextStyle(fontSize: 18),
              ),
            ],
          ),
        ),
        floatingActionButton: FloatingActionButton(
          onPressed: controller.incrementCounter,
          tooltip: 'Increment',
          child: Icon(Icons.add),
        ),
      ),
    );
  }
}
