import 'package:flutter/material.dart';
import 'package:fluttered/architecture.dart';

import '../controllers/pvc_demo_mutable_controller.dart';

/// With this page we are taking a different approach of
/// creating the page using the controller.
///
/// Instead of creating a widget only to define the `StatefulMutablePVC`
/// widget, we have created a method named `statefulMutablePage`, This
/// method then creates the `StatefulMutablePVC` directly.
final statefulMutablePage = () => StatefulMutablePvc<PVCDemoMutableController>(
      controller: PVCDemoMutableController(),
      onModelReady: (controller) => controller.onInitView(),
      builder: (context, controller, child) => Scaffold(
        appBar: AppBar(
          title: Text('StatefulMutablePVC'),
        ),
        body: Center(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Text(
                'Notice the counter value starts with 55 instead of 0. This is because we are invoking the "controller.onInitView()" method which sets the initial value of "counter" to 55, by passing "controller.onInitView()" as "onModelReady" parameter.',
                style: TextStyle(fontSize: 18),
              ),
              SizedBox(height: 100),
              Text(
                'You have pushed the button this many times:',
                style: TextStyle(fontSize: 22),
              ),
              Text(
                '${controller.counter}',
                style: Theme.of(context).textTheme.display1,
              ),
              SizedBox(height: 100),
              Text(
                'Tapping the button is updating the UI with the new "counter" value. This is because, "StatelessMutablePVC" creates a "ChangeNotifierProvider". A "ChangeNotifierProvider" notifies it\'s consumers about changes, and thus the widget tree rebuild itself.',
                style: TextStyle(fontSize: 18),
              ),
            ],
          ),
        ),
        floatingActionButton: FloatingActionButton(
          onPressed: controller.incrementCounter,
          tooltip: 'Increment',
          child: Icon(Icons.add),
        ),
      ),
    );
